#ifndef DataTypesH
#define DataTypesH
//---------------------------------------------------------------------------
/**
* @file DataTypes.h
 Here is a universal place to define model-specific data types, structures,
 enums, etc.

 Copyright 2003 Charles D. Canham.
 @author Lora E. Murphy

 <br>Edit history:
 <br>-----------------
 <br>April 28, 2004 - Submitted as beta (LEM)
*/

 /**
 Structure for holding species-specific float values. An object which does not
 apply to all species does not need to have data for all species for those data
 that are species-specific.  (Besides, that data may not exist in the first
 place.)  This allows bundling of a value with species code to which it applies.

 The naming convention rules are being ignored for these so that they can all
 use the same internal names - making coding and type-changing easier.*/
 struct floatVal {float val; /**<Value.*/
                  short int code; /**<Species code.*/
                  };

 /**
 Structure for holding species-specific integer values. An object which does not
 apply to all species does not need to have data for all species for those data
 that are species-specific.  (Besides, that data may not exist in the first
 place.)  This allows bundling of a value with species code to which it applies.

 The naming convention rules are being ignored for these so that they can all
 use the same internal names - making coding and type-changing easier.*/
 struct intVal {int val; /**<Value.*/
                short int code; /**<Species code.*/
                };

 /**
 Structure for holding species-specific double values. An object which does not
 apply to all species does not need to have data for all species for those data
 that are species-specific.  (Besides, that data may not exist in the first
 place.)  This allows bundling of a value with species code to which it applies.

 The naming convention rules are being ignored for these so that they can all
 use the same internal names - making coding and type-changing easier.*/
 struct doubleVal {double val; /**<Value.*/
                short int code; /**<Species code.*/
                };

 /**
 Structure for holding species-specific boolean values. An object which does not
 apply to all species does not need to have data for all species for those data
 that are species-specific.  (Besides, that data may not exist in the first
 place.)  This allows bundling of a value with species code to which it applies.

 The naming convention rules are being ignored for these so that they can all
 use the same internal names - making coding and type-changing easier.*/
 struct boolVal {bool val; /**<Value.*/
                short int code; /**<Species code.*/
                };

 /**Holds species/type combos.*/
 struct stcSpeciesTypeCombo {
   short int iSpecies,  /**<Species number.  Matches tree population's list.*/
             iType; /**<Type number.  Matches tree population's list.*/
             };


 /**List of file type codes.*/
 enum fileType{ notrecognized = -1, /**<Unrecognized file type*/
                 oldsortie = 0, /**<File from old version of SORTIE*/
                 parfile = 1,   /**<Parameter file*/
                 tree,          /**<General tree file*/
                 treemap,       /**<Tree map file only*/
                 batchfile,     /**<Batch file*/
                 map,           /**<Grid map*/
                 detailed_output,       /**<Detailed output file*/
                 detailed_output_timestep, /**<Detailed output timestep file*/
                 lastfile       /**<Placeholder*/
               };

//---------------------------------------------------------------------------
#endif
