//---------------------------------------------------------------------------
#include "PlatformFuncs.h"
#include <stdio.h>
#include <io.h>
#include "Constants.h"
#include "Messages.h"
#include <windows.h>
#include <shellapi.h>
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////
// TarballSetup
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// May 3, 2004 - Changed paths to "bin\\x" (LEM)
// November 11, 2004 - Added app path argument (LEM)
////////////////////////////////////////////////////////////////////////////
bool TarballSetup(char *cAppPath) {
  char *cPath = new char[strlen(cAppPath) + 10];
  strcpy(cPath, cAppPath);
  strcat(cPath, "gzip.exe");
  int iAvailable = access(cPath, 0); //0 signifies checking for existence
  if (0 == iAvailable) {
    strcpy(cPath, cAppPath);
    strcat(cPath, "tar.exe");
//    iAvailable = access("bin\\tar.exe", 0);
    if (0 == iAvailable) {
      delete[] cPath;
      return true;
    } else {
      delete[] cPath;
      return false;
    }
  } else
    delete[] cPath;
    return false;
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////
// ZipFile
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// May 3, 2004 - Changed paths to "bin\\x" (LEM)
// November 11, 2004 - Added app path argument (LEM)
////////////////////////////////////////////////////////////////////////////
void ZipFile(char *cFileName, char *cAppPath) {
  SHELLEXECUTEINFO *params;       //the data structure for running ShellExecuteEx
  FILE *filetest;  //only this kind of file can be tested for existence of a
                   //file without creating

//  char cGzipPath[] = "bin\\gzip.exe",
  char *cGzipPath = new char[strlen(cAppPath) + 10],
       *cExecDir,
       *cParamString,
       cFlags[] = "-f -q "; //gzip flags - -f is force, which will overwrite
                            //existing files, and -q is quiet, which suppresses
                            //warnings

  strcpy(cGzipPath, cAppPath);
  strcat(cGzipPath, "gzip.exe");

  //Does the file exist?  If not, exit without error
  filetest = fopen(cFileName, "r");
  if (filetest == NULL) return;
  else fclose(filetest);

  params = new SHELLEXECUTEINFO;
  //Declare a string to hold the parameters - the + 3 is for two double quotes
  //and the ending NULL character
  cParamString = new char[strlen(cFileName) + strlen(cFlags) + 3];
  sprintf(cParamString, "%s%s%s%s", cFlags, "\"", cFileName, "\"");

  //Get the current directory
  cExecDir = _getcwd( NULL, 0 );
  if (NULL == cExecDir) {
  	modelErr stcErr;
  	stcErr.iErrorCode = UNKNOWN;
    strcpy(stcErr.cFunction, "ZipFile");
    strcpy(stcErr.cMoreInfo, "Could not use _getcwd to get current directory.");
    throw(stcErr);
  }

  //Write the data structure which controls GZIP's execution
  params->cbSize = sizeof(SHELLEXECUTEINFO); //size of this structure
  params->fMask = SEE_MASK_NOCLOSEPROCESS;  //flag to allow us to find out when
      //the process terminates
  params->lpVerb = 0; //what GZIP should do - open (run)
  params->lpFile = cGzipPath; //path to gzip executable
  params->lpParameters = cParamString; //parameters passed to gzip
  params->lpDirectory = cExecDir;
//  params->nShow = SW_NORMAL;
  params->nShow = SW_HIDE; //show no DOS window

  //Run gzip
  if (ShellExecuteEx(params))
    //force zip to complete before moving on
    WaitForSingleObject(params->hProcess, INFINITE);

  delete[] cParamString;
  delete[] cGzipPath;
  delete params;
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////
// UnzipFile
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// May 3, 2004 - Changed paths to "bin\\x" (LEM)
////////////////////////////////////////////////////////////////////////////
/*void UnzipFile(char *cFileName) {
  SHELLEXECUTEINFO *params;       //the data structure for running ShellExecuteEx
  FILE *filetest;  //only this kind of file can be tested for existence of a
                   //file without creating
  char cGzipPath[] = "bin\gzip.exe",
       //cExecDir[] = "c:\\sortie\\code\\new model\\",
       cExecDir[MAXDIR],
       *cParamString,
       cFlags[] = "-f -q -d "; //gzip flags - -f is force, which will overwrite
                            //existing files, and -q is quiet, which suppresses
                            //warnings, and -d is the uncompress flag

  //Does the file exist?  If not, exit without error
  filetest = fopen(cFileName, "r");
  if (filetest == NULL) return;
  else fclose(filetest);

  params = new SHELLEXECUTEINFO;
  //Declare a string to hold the parameters - the + 3 is for two double quotes
  //and the ending NULL character
  cParamString = new char[strlen(cFileName) + strlen(cFlags) + 3];
  sprintf(cParamString, "%s%s%s%s", cFlags, "\"", cFileName, "\"");

  //Get the current directory
  getcurdir(0, cExecDir);

  //Write the data structure which controls GZIP's execution
  params->cbSize = sizeof(SHELLEXECUTEINFO); //size of this structure
  params->fMask = SEE_MASK_NOCLOSEPROCESS;  //flag to allow us to find out when
      //the process terminates
  params->lpVerb = 0; //what GZIP should do - open (run)
  params->lpFile = cGzipPath; //path to gzip executable
  params->lpParameters = cParamString; //parameters passed to gzip
  params->lpDirectory = cExecDir;
  //params->nShow = SW_NORMAL;
  params->nShow = SW_HIDE; //show no DOS window

  //Run gzip
  if (ShellExecuteEx(params))
    //force zip to complete before moving on
    WaitForSingleObject(params->hProcess, INFINITE);

  delete[] cParamString;
  delete params;
} */
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////
// AddFileToTarball
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// May 3, 2004 - Changed paths to "bin\\x" (LEM)
// November 11, 2004 - Added app path argument (LEM)
////////////////////////////////////////////////////////////////////////////
void AddFileToTarball(char *cTarball, char *cFileToAdd, char *cAppPath) {
  SHELLEXECUTEINFO *params;  //the data structure for running ShellExecuteEx
  FILE *filetest;  //only this kind of file can be tested for existence of a
                   //file without creating
 // std::string cParamString;
  char //cTarPath[] = "bin\\tar.exe",
       *cTarPath = new char[strlen(cAppPath) + 10],
       *cExecDir,
       *cParamString,
       *cSearcher,
       *cPos,
       *cParamPos,
       *cBufferPos,
       *cBuffer,
       cFlags[35];
  int iStringLength;

  //Does the file to add exist?  If not, exit without error
  filetest = fopen(cFileToAdd, "r");
  if (filetest == NULL) return;
  else fclose(filetest);

  strcpy(cTarPath, cAppPath);
  strcat(cTarPath, "tar.exe");

  //Set up the parameters for tar based on whether or not the tarball already
  //exists
  filetest = fopen(cTarball, "r");
  if (filetest == NULL) {

    //The tarball doesn't exist - set up tar's flags to create
    //strcpy(cFlags, "--create --no-recursion --file ");   //-c means create - -f for file name
    strcpy(cFlags, "-cf ");

  } else {

    //The tarball does exist - set up tar's flags to create
    //strcpy(cFlags, "--append --file ");  //-r means append - -f for file name
    strcpy(cFlags, "-rf ");
    fclose(filetest);
  }

  //Set up the arguments to pass to tar
  iStringLength = strlen(cFileToAdd)  //Filename
                             + strlen(cTarball)  //Tarball filename
                             + strlen(cFlags) //Flags
                             + 26;

  cParamString = new char[iStringLength];
  cBuffer = new char[iStringLength];
//  cParamString << cFlags << "\"" << cTarball << "\" \"" << cFileToAdd << "\"";

//  cParamString.replace(.find("middle",0), 6, "center");

  sprintf(cBuffer, "%s%s%s%s%s%s", cFlags, "\"", cTarball, "\" \"",
         cFileToAdd, "\"");

  //We have to double all the slashes to avoid escape character sequences
  //Search for single slashes
  cParamPos = cParamString;
  cBufferPos = cBuffer;
  cSearcher = strstr(cBufferPos, "\\");
  while (cSearcher) {

    //Copy the string up until the new position into cParamString
    for (cPos = cBufferPos; cPos < cSearcher; cPos++) {
      *cParamPos = *cPos;
      cParamPos++;
    }

    //Add two slashes
    *cParamPos = '\\';
    cParamPos++; *cParamPos = '\\';

    //Advance the position of the iterators
    cParamPos++;
    cBufferPos = cSearcher+1;

    cSearcher = strstr(cBufferPos, "\\");
  }
  *cParamPos = '\0';
  strcat(cParamString, cBufferPos);

  if (cParamString[0] == '\0') {
    strcpy(cParamString, cBuffer);
  }

  params = new SHELLEXECUTEINFO;

  //Get the current directory - not declaring the butter ahead of time forces
  //_getcwd to do it
  cExecDir = _getcwd( NULL, 0 );
  if (NULL == cExecDir) {
  	modelErr stcErr;
  	stcErr.iErrorCode = UNKNOWN;
    strcpy(stcErr.cFunction, "AddFileToTarball");
    strcpy(stcErr.cMoreInfo, "Could not use _getcwd to get current directory.");
    throw(stcErr);
  }

  //Write the data structure which controls tar's execution
  params->cbSize = sizeof(SHELLEXECUTEINFO); //size of this structure
  params->fMask = SEE_MASK_NOCLOSEPROCESS;  //flag to allow us to find out when
      //the process terminates
  params->lpVerb = 0; //what TAR should do - open (run)
  params->lpFile = cTarPath; //path to tar executable
  params->lpParameters = cParamString; //parameters passed to tar
  params->lpDirectory = cExecDir;
//  params->nShow = SW_NORMAL;
  params->nShow = SW_HIDE; //show no DOS window

  //Run tar
  if (ShellExecuteEx(params)) {
    //force tar to complete before moving on
    WaitForSingleObject(params->hProcess, INFINITE);
  }

  delete[] cParamString;
  delete params;
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////
// AddFileToTarball
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// May 3, 2004 - Changed paths to "bin\\x" (LEM)
// November 11, 2004 - Added app path argument (LEM)
////////////////////////////////////////////////////////////////////////////
void AddFileToNewTarball(char *cTarball, char *cFileToAdd, char *cAppPath) {
  SHELLEXECUTEINFO *params;  //the data structure for running ShellExecuteEx
  FILE *filetest;  //only this kind of file can be tested for existence of a
                   //file without creating
  char //cTarPath[] = "bin\\tar.exe",
       *cTarPath = new char[strlen(cAppPath) + 10],
       *cExecDir,
       *cParamString,
       *cSearcher,
       *cPos,
       *cParamPos,
       *cBufferPos,
       *cBuffer,
       cFlags[35];
  int iStringLength;

  //Does the file to add exist?  If not, exit without error
  filetest = fopen(cFileToAdd, "r");
  if (filetest == NULL) return;
  else fclose(filetest);

  strcpy(cTarPath, cAppPath);
  strcat(cTarPath, "tar.exe");

  //Set up the parameters for tar to create
  strcpy(cFlags, "-cf "); //-c means create - -f for file name

  //Set up the arguments to pass to tar
  iStringLength = strlen(cFileToAdd)  //Filename
                             + strlen(cTarball)  //Tarball filename
                             + strlen(cFlags) //Flags
                             + 26;

  cParamString = new char[iStringLength];
  cBuffer = new char[iStringLength];

  sprintf(cBuffer, "%s%s%s%s%s%s", cFlags, "\"", cTarball, "\" \"",
         cFileToAdd, "\"");

  //We have to double all the slashes to avoid escape character sequences
  //Search for single slashes
  cParamPos = cParamString;
  cBufferPos = cBuffer;
  cSearcher = strstr(cBufferPos, "\\");
  while (cSearcher) {

    //Copy the string up until the new position into cParamString
    for (cPos = cBufferPos; cPos < cSearcher; cPos++) {
      *cParamPos = *cPos;
      cParamPos++;
    }

    //Add two slashes
    *cParamPos = '\\';
    cParamPos++; *cParamPos = '\\';

    //Advance the position of the iterators
    cParamPos++;
    cBufferPos = cSearcher+1;

    cSearcher = strstr(cBufferPos, "\\");
  }
  *cParamPos = '\0';
  strcat(cParamString, cBufferPos);

  if (cParamString[0] == '\0') {
    strcpy(cParamString, cBuffer);
  }

  params = new SHELLEXECUTEINFO;

  //Get the current directory - not declaring the butter ahead of time forces
  //_getcwd to do it
  cExecDir = _getcwd( NULL, 0 );
  if (NULL == cExecDir) {
  	modelErr stcErr;
  	stcErr.iErrorCode = UNKNOWN;
    strcpy(stcErr.cFunction, "AddFileToNewTarball");
    strcpy(stcErr.cMoreInfo, "Could not use _getcwd to get current directory.");
    throw(stcErr);
  }

  //Write the data structure which controls tar's execution
  params->cbSize = sizeof(SHELLEXECUTEINFO); //size of this structure
  params->fMask = SEE_MASK_NOCLOSEPROCESS;  //flag to allow us to find out when
      //the process terminates
  params->lpVerb = 0; //what TAR should do - open (run)
  params->lpFile = cTarPath; //path to tar executable
  params->lpParameters = cParamString; //parameters passed to tar
  params->lpDirectory = cExecDir;
//  params->nShow = SW_NORMAL;
  params->nShow = SW_HIDE; //show no DOS window

  //Run tar
  if (ShellExecuteEx(params)) {
    //force tar to complete before moving on
    WaitForSingleObject(params->hProcess, INFINITE);
  }

  delete[] cParamString;
  delete params;
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////
// DeleteThisFile
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
void DeleteThisFile(char *cFile) {

  //DeleteFile(cFile);
  //While using the above code is preferable, for some reason Java crashes when
  //loading the DLL.  So we have to do things the hard way - through the shell.

  //This next part I'm copying from http://www.bridgespublishing.com/articles/
  //issues/9806/File_operations.htm
  // Create a char array and fill it with nulls.  This is because pFrom and
  //pTo have to be double-null terminated, so this will make sure of it.
  char src[MAX_PATH], dst[MAX_PATH];
  memset(src, 0, sizeof(src));
  memset(dst, 0, sizeof(dst));
  // Create a SHFILOPSTRUCT and zero it, too.
  SHFILEOPSTRUCT fos;
  memset(&fos, 0, sizeof(fos));

  //Set the fields of SHFILOPSTRUCT
  strcpy(src, cFile);


  fos.hwnd = NULL; //handle of parent window
  fos.wFunc = FO_DELETE; //operation to perform
  fos.pFrom = src; //pFrom - file to work on
  fos.pTo = dst; //pTo - destination
  fos.fFlags = FOF_SILENT | FOF_NOCONFIRMATION; //no dialog box and no asking
       //the user

  //Now perform the operation
  SHFileOperation(&fos);

}

////////////////////////////////////////////////////////////////////////////
// DoesFileExist
// Edit history:
// -------------
// September 26, 2006 - Created (LEM)
////////////////////////////////////////////////////////////////////////////
bool DoesFileExist(char *cFile) {
  int iCode = access(cFile, 0); //0 signifies checking for existence
  if (0 == iCode) 
    return true;
  else
    return false;
}

////////////////////////////////////////////////////////////////////////////
// LaunchProcess
// Edit history:
// -------------
// September 26, 2006 - Created (LEM)
////////////////////////////////////////////////////////////////////////////
void LaunchProcess(char *cFile, char *cArgs, char *cDir) {
  SHELLEXECUTEINFO *params;       //the data structure for running ShellExecuteEx
  
  params = new SHELLEXECUTEINFO;
  params->cbSize = sizeof(SHELLEXECUTEINFO); //size of this structure
  params->fMask = SEE_MASK_NOCLOSEPROCESS;  //flag to allow us to find out when
    //the process terminates
  params->lpVerb = 0; //what the executable should do - open (run)
  params->lpFile = cFile; //path to executable
  params->lpParameters = cArgs; //parameters passed
  params->lpDirectory = cDir;
  params->nShow = SW_HIDE;
        
  //Run the executable
  if (ShellExecuteEx(params)) {
    //force the process to complete before moving on
    WaitForSingleObject(params->hProcess, INFINITE);
  }
}
