//---------------------------------------------------------------------------

#ifndef PlatformFuncsH
#define PlatformFuncsH
//---------------------------------------------------------------------------

/**
* @file PlatformFuncs.h
  These are functions which use platform-specific code.

  Copyright 2003 Charles D. Canham.
  @author Lora E. Murphy

  <br>Edit history:
  <br>-----------------
  <br>April 28, 2004 - Submitted as beta (LEM)
*/

/**
  Verifies that gzip and tar are where we expect them to be (in the
  same directory as the executable).  This should be called before either of the functions
  below and a fatal error thrown if it returns false.

  @param cAppPath Path to the main application, ending with the path separator.
  The executables are expected to be in the same path.
  @return true if the executables are where they should be.  Otherwise, false.
*/
 bool TarballSetup(char *cAppPath);

/**
  Zips a file with GZIP.  If the file does not exist, the function does nothing.

  @param cFileName File to zip.
  @param cAppPath Path to the main application, ending with the path separator.
  The GZIP executable is expected to be in the same path.
*/
 void ZipFile(char *cFileName, char *cAppPath);

/**
* Adds a file to the end of a tarball.  If the tarball does not exist, it will
* be created.  The file to add can be either zipped or not zipped.
*
* @param cTarball File name of tarball
* @param cFileToAdd File name of the file to add to the tarball
* @param cAppPath Path to the main application, ending with the path separator.
* The TAR executable is expected to be in the same path.
*/
 void AddFileToTarball(char *cTarball, char *cFileToAdd, char *cAppPath);

 /**
* Creates a new tarball and adds a file to it.  Any existing tarball of that
* name will be overwritten.  If the tarball does not exist, it will
* be created.  The file to add can be either zipped or not zipped.
*
* @param cTarball File name of tarball
* @param cFileToAdd File name of the file to add to the tarball
* @param cAppPath Path to the main application, ending with the path separator.
* The TAR executable is expected to be in the same path.
*/
 void AddFileToNewTarball(char *cTarball, char *cFileToAdd, char *cAppPath);


 /**
  * Deletes a file.
  * @param cFile File name of the file to delete
  */
 void DeleteThisFile(char *cFile);
 
 /**
  * Checks for the existence of a file
  * @param cFile File name of the file to check for existence
  * @return True if the file exists, false if it doesn't
  */
 bool DoesFileExist(char *cFile);
 
 /**
  * Launches a process and waits for it to finish.
  * @param cFile Executable to launch
  * @param cArgs Arguments to pass to process
  * @param cDir Working directory
  */
 void LaunchProcess(char *cFile, char *cArgs, char *cDir);

//---------------------------------------------------------------------------
#endif
